<?php

namespace FiloBlu\AdvancedCustomOptions\Model\ResourceModel\Product\Option;

use Magento\Directory\Model\CurrencyFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Filesystem;
use Magento\Framework\Model\AbstractModel;
use Magento\Framework\Model\ResourceModel\Db\AbstractDb;
use Magento\Framework\Model\ResourceModel\Db\Context;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Class Value
 * @package FiloBlu\AdvancedCustomOptions\Model\ResourceModel\Product\Option
 */
class Value extends \Magento\Catalog\Model\ResourceModel\Product\Option\Value
{
    /**
     * @var Filesystem
     */
    protected $filesystem;

    /**
     * @var string
     */
    protected $tmpImagePath = '/filoblu/advanced_custom_options/tmp/custom_option/';

    /**
     * @var string
     */
    protected $imagePath = '/filoblu/advanced_custom_options/custom_option/';

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * Value constructor.
     * @param Context $context
     * @param CurrencyFactory $currencyFactory
     * @param StoreManagerInterface $storeManager
     * @param ScopeConfigInterface $config
     * @param null $connectionName
     * @param Filesystem $filesystem
     * @param LoggerInterface $logger
     */
    public function __construct(
        Context $context,
        CurrencyFactory $currencyFactory,
        StoreManagerInterface $storeManager,
        ScopeConfigInterface $config,
        $connectionName = null,
        Filesystem $filesystem,
        LoggerInterface $logger
    ) {
        $this->filesystem = $filesystem;
        $this->logger = $logger;
        parent::__construct($context, $currencyFactory, $storeManager, $config, $connectionName);
    }

    /**
     * @param AbstractModel $object
     * @return AbstractDb
     */
    protected function _afterSave(AbstractModel $object)
    {
        $this->_saveValueImages($object);
        $this->_saveValueConfigurations($object);

        return parent::_afterSave($object);
    }

    /**
     * @param AbstractModel $object
     */
    protected function _saveValueImages(AbstractModel $object)
    {
        //IF THERE IS OPTION IMAGE WE SAVE THAT VALUE OTHERWISE WE SAVE NULL
        $optionImage = $object->getOptionImage();
        $optionImageName = null;
        if (count($optionImage)) {
            // Fix for import
            if (is_string($optionImage)) {
                $optionImageName = $optionImage;
            } else {
                if (isset($optionImage[0]['name'])) {
                    //TODO: controllare il save per immagini vuote
                    $optionImageName = $optionImage[0]['name'];
                    try {
                        $res = $this->moveFileFromTmp($optionImageName, $object);
                        if ($res != null) {
                            $optionImageName = $res;
                        }
                    } catch (\Exception $e) {
                        $this->logger->error($e);
                    }
                }
            }
        }

        $imageTable = $this->getTable('catalog_product_option_type_image');

        if (true) {
            //save for store_id = 0
            $select = $this->getConnection()->select()->from(
                $imageTable,
                'option_type_id'
            )->where(
                'option_type_id = ?',
                (int)$object->getId()
            )->where(
                'store_id = ?',
                Store::DEFAULT_STORE_ID
            );
            $optionTypeId = $this->getConnection()->fetchOne($select);

            if ($optionTypeId) {
                if ($object->getStoreId() == '0') {
                    $bind = ['option_image' => $optionImageName];
                    $where = [
                        'option_type_id = ?' => $optionTypeId,
                        'store_id = ?' => Store::DEFAULT_STORE_ID,
                    ];

                    $this->getConnection()->update($imageTable, $bind, $where);
                }
            } else {
                $bind = [
                    'option_type_id' => (int)$object->getId(),
                    'store_id' => Store::DEFAULT_STORE_ID,
                    'option_image' => $optionImageName
                ];
                $this->getConnection()->insert($imageTable, $bind);
            }
        }
    }

    /**
     * @param AbstractModel $object
     */
    protected function _saveValueConfigurations(AbstractModel $object)
    {
        //IF THERE IS OPTION IMAGE WE SAVE THAT VALUE OTHERWISE WE SAVE NULL
        $optionSalable = $object->getOptionSalable();
        $optionDefault = $object->getOptionDefault();

        $configurationTable = $this->getTable('catalog_product_option_type_configuration');

        if (true) {
            //save for store_id = 0
            $select = $this->getConnection()->select()->from(
                $configurationTable,
                'option_type_id'
            )->where(
                'option_type_id = ?',
                (int)$object->getId()
            )->where(
                'store_id = ?',
                Store::DEFAULT_STORE_ID
            );
            $optionTypeId = $this->getConnection()->fetchOne($select);

            if ($optionTypeId) {
                if ($object->getStoreId() == '0') {
                    $bind = [
                        'option_salable' => $optionSalable,
                        'option_default' => $optionDefault
                    ];
                    $where = [
                        'option_type_id = ?' => $optionTypeId,
                        'store_id = ?' => Store::DEFAULT_STORE_ID,
                    ];

                    $this->getConnection()->update($configurationTable, $bind, $where);
                }
            } else {
                $bind = [
                    'option_type_id' => (int)$object->getId(),
                    'store_id' => Store::DEFAULT_STORE_ID,
                    'option_salable' => $optionSalable,
                    'option_default' => $optionDefault
                ];
                $this->getConnection()->insert($configurationTable, $bind);
            }
        }
    }

    /**
     * @param $img
     * @return int
     * @throws FileSystemException
     */
    protected function moveFileFromTmp($img, $object)
    {
        $filename = null;
        //Check if media exists in tmp folder otherwise return null
        if ($this->filesystem->getDirectoryRead(DirectoryList::MEDIA)->isExist($this->tmpImagePath . $img)) {
            $content = $this->filesystem->getDirectoryRead(DirectoryList::MEDIA)->readFile($this->tmpImagePath . $img);
            if (!empty($content)) {
                $ext = pathinfo($img, PATHINFO_EXTENSION);
                $filename = $img;
                $this->filesystem->getDirectoryWrite(DirectoryList::MEDIA)->writeFile($this->imagePath . $filename, $content);
                $this->filesystem->getDirectoryWrite(DirectoryList::MEDIA)->delete($this->tmpImagePath . $img);
            }
        }
        return $filename;
    }
}
