<?php

namespace FiloBlu\AdvancedCustomOptions\Ui\DataProvider\Catalog\Product\Form\Modifier;

use Magento\Catalog\Model\Config\Source\Product\Options\Price as ProductOptionsPrice;
use Magento\Catalog\Model\Locator\LocatorInterface;
use Magento\Catalog\Model\ProductOptions\ConfigInterface;
use Magento\Catalog\Ui\DataProvider\Product\Form\Modifier\AbstractModifier;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\File\Mime;
use Magento\Framework\Filesystem;
use Magento\Framework\Stdlib\ArrayManager;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Ui\Component\Form\Element\Checkbox;
use Magento\Ui\Component\Form\Element\DataType\Boolean;
use Magento\Ui\Component\Form\Element\DataType\Text;
use Magento\Ui\Component\Form\Element\Input;
use Magento\Ui\Component\Form\Element\Select;
use Magento\Ui\Component\Form\Field;

/**
 * Class CustomOptions
 * @package FiloBlu\AdvancedCustomOptions\Ui\DataProvider\Catalog\Product\Form\Modifier
 */
class CustomOptions extends AbstractModifier
{
    /**
     * @var RequestInterface
     */
    protected $request;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var Mime
     */
    protected $mime;

    /**
     * @var Filesystem
     */
    protected $filesystem;

    /**
     * @var string
     */
    public $mediaFolderRelativePath = '/filoblu/advanced_custom_options/custom_option/';
    /**
     * @var \Magento\Catalog\Ui\DataProvider\Product\Form\Modifier\CustomOptions
     */
    protected $customOptions;
    /**
     * @var ConfigInterface
     */
    protected $productOptionsConfig;
    /**
     * @var LocatorInterface
     */
    protected $locator;
    /**
     * @var ProductOptionsPrice
     */
    protected $productOptionsPrice;
    /**
     * @var UrlInterface
     */
    protected $urlBuilder;
    /**
     * @var ArrayManager
     */
    protected $arrayManager;

    /**
     * CustomOptions constructor.
     * @param LocatorInterface $locator
     * @param StoreManagerInterface $storeManager
     * @param ConfigInterface $productOptionsConfig
     * @param ProductOptionsPrice $productOptionsPrice
     * @param UrlInterface $urlBuilder
     * @param ArrayManager $arrayManager
     * @param RequestInterface $request
     * @param Mime $mime
     * @param Filesystem $filesystem
     * @param \Magento\Catalog\Ui\DataProvider\Product\Form\Modifier\CustomOptions $customOptions
     */
    public function __construct(
        LocatorInterface $locator,
        StoreManagerInterface $storeManager,
        ConfigInterface $productOptionsConfig,
        ProductOptionsPrice $productOptionsPrice,
        UrlInterface $urlBuilder,
        ArrayManager $arrayManager,
        RequestInterface $request,
        Mime $mime,
        Filesystem $filesystem,
        \Magento\Catalog\Ui\DataProvider\Product\Form\Modifier\CustomOptions $customOptions
    ) {
        $this->productOptionsPrice = $productOptionsPrice;
        $this->urlBuilder = $urlBuilder;
        $this->arrayManager = $arrayManager;
        $this->locator = $locator;
        $this->request = $request;
        $this->storeManager = $storeManager;
        $this->mime = $mime;
        $this->filesystem = $filesystem;
        $this->customOptions = $customOptions;
        $this->productOptionsConfig = $productOptionsConfig;
        //parent::__construct($locator, $storeManager, $productOptionsConfig, $productOptionsPrice, $urlBuilder, $arrayManager);
    }

    /**
     * Field option code
     */
    const FIELD_CODE_NAME = 'code';

    /**
     * Field Option Image
     */
    const FIELD_OPTION_IMAGE_NAME = 'option_image';

    /**
     * Field Option Salable
     */
    const FIELD_OPTION_SALABLE_NAME = 'option_salable';

    /**
     * Field Option Default
     */
    const FIELD_OPTION_DEFAULT_NAME = 'option_default';

    /**
     * FB Image Group
     */
    const FB_IMAGE_GROUP = 'filoblu_image_group';

    /**
     * FiloBlu Custom Indexes
     *
     * NB: Do not change
     */
    const FB_CUSTOM_INDEX_IMAGE = "filoblu_custom_image";

    /**
     * FB image relative path
     */
    const FB_RELATIVE_IMG_PATH = '/filoblu/advanced_custom_options/custom_option/';

    /**
     * Image Field
     *
     * @param $sortOrder
     * @return array
     */
    protected function getOptionImageFieldConfig($sortOrder)
    {
        return [
            'arguments' => [
                'data' => [
                    'config' => [
                        'label' => __('Image'),
                        'componentType' => 'imageUploader',
                        'formElement' => 'imageUploader',
                        'placeholderType' => 'image',
                        'source' => 'image',
                        'dataType' => 'string',
                        'visible' => true,
                        'elementTmpl' => 'ui/form/element/uploader/image',
                        'previewTmpl' => 'ui/form/element/uploader/preview',
                        'showFallabackReset' => true,
                        'validation' => [
                            'required-entry' => false
                        ],
                        'uploaderConfig' => [
                            'url' => $this->urlBuilder->addSessionParam()->getUrl(
                                'advanced_custom_options/product/preview/field/' . self::FIELD_OPTION_IMAGE_NAME
                            ),
                            'paramName' => self::FIELD_OPTION_IMAGE_NAME
                        ],
                        'sortOrder' => $sortOrder,
                    ]
                ],
            ]
        ];
    }

    /**
     * Salable Field
     *
     * @param $sortOrder
     * @return array
     */
    protected function getOptionSalableConfig($sortOrder)
    {
        return [
            'arguments' => [
                'data' => [
                    'config' => [
                        'label' => __('Salable'),
                        'componentType' => Field::NAME,
                        'formElement' => Checkbox::NAME,
                        'dataScope' => self::FIELD_OPTION_SALABLE_NAME,
                        'dataType' => Boolean::NAME,
                        'sortOrder' => $sortOrder,
                        'prefer' => 'toggle',
                        'valueMap' => ["true" => "1","false" => "0"],
                        'default' => "1"
                    ],
                ],
            ],
        ];
    }

    /**
     * @param $sortOrder
     * @return array
     */
    protected function getOptionCodeConfig($sortOrder)
    {
        return [
            'arguments' => [
                'data' => [
                    'config' => [
                        'label' => __('Option Code'),
                        'componentType' => Field::NAME,
                        'formElement' => Input::NAME,
                        'dataScope' => self::FIELD_CODE_NAME,
                        'dataType' => Text::NAME,
                        'sortOrder' => $sortOrder,
                        'disabled' => false,
                        'validation' => [
                            'required-entry' => false
                        ]
                    ],
                ],
            ],
        ];
    }

    /**
     * Default Field
     *
     * @param $sortOrder
     * @return array
     */
    protected function getOptionDefaultConfig($sortOrder)
    {
        return [
            'arguments' => [
                'data' => [
                    'config' => [
                        'label' => __('Default'),
                        'componentType' => Field::NAME,
                        'formElement' => Checkbox::NAME,
                        'dataScope' => self::FIELD_OPTION_DEFAULT_NAME,
                        'dataType' => Boolean::NAME,
                        'sortOrder' => $sortOrder,
                        'prefer' => 'toggle',
                        'valueMap' => ["true" => "1","false" => "0"],
                        'default' => "0"
                    ],
                ],
            ],
        ];
    }

    /**
     * Returns image url
     *
     * @param $image
     * @return bool|string
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function getImageUrl($image)
    {
        $url = false;
        if ($image) {
            if (is_string($image)) {
                $store = $this->storeManager->getStore();

                $isRelativeUrl = substr($image, 0, 1) === '/';

                $mediaBaseUrl = $store->getBaseUrl(
                    \Magento\Framework\UrlInterface::URL_TYPE_MEDIA
                );

                if ($isRelativeUrl) {
                    $url = $image;
                } else {
                    $url = rtrim($mediaBaseUrl, '/')
                        . $this->mediaFolderRelativePath
                        . $image;
                }
            } else {
                throw new LocalizedException(
                    __('Something went wrong while getting the image url.')
                );
            }
        }
        return $url;
    }

    /**
     * @param $fileName
     * @return string
     */
    public function getImagePath($fileName)
    {
        $mediapath = rtrim($this->filesystem->getDirectoryRead(DirectoryList::MEDIA)->getAbsolutePath(), '/');
        $result = $mediapath . $this->mediaFolderRelativePath . $fileName;
        return $result;
    }

    /**
     * @param $fileName
     * @return string
     */
    public function getMimeType($fileName)
    {
        $filePath = $this->getImagePath($fileName);
        $result = $this->mime->getMimeType($filePath);
        return $result;
    }

    /**
     * Get file statistics data
     *
     * @param string $fileName
     * @return array
     */
    public function getStat($fileName)
    {
        $filePath = $this->getImagePath($fileName);
        $result = $this->filesystem->getDirectoryRead(DirectoryList::MEDIA)->stat($this->mediaFolderRelativePath . $fileName);

        return ((count($result) && isset($result['size'])) ? $result['size'] : 0);
    }

    /**
     * @param array $meta
     * @return array
     */
    public function modifyMeta(array $meta)
    {
        
        
        /* TEMP: enable this condition to exclude product entities  */
        // if ($this->locator->getProduct()->getTypeId() != 'simple' && $this->locator->getProduct()->getTypeId() != 'configurable') {
        //     return $meta;
        // }

        try {
            $meta['custom_options']["children"]["options"]["children"]["record"]["children"]["container_option"]["children"]["container_common"]["children"]["type"]["arguments"]["data"]["config"]["groupsConfig"][self::FB_IMAGE_GROUP] = [
                'values' => [self::FIELD_OPTION_IMAGE_NAME],
                'indexes' => [
                    self::FB_CUSTOM_INDEX_IMAGE
                ]
            ];

            $meta['custom_options']["children"]["options"]["children"]["record"]["children"]["container_option"]["children"]["container_common"]["children"][self::FIELD_CODE_NAME] = $this->getOptionCodeConfig(31);
            $meta['custom_options']["children"]["options"]["children"]["record"]["children"]["container_option"]["children"]["container_common"]["children"]["type"]["arguments"]["data"]["config"]["component"] = 'FiloBlu_AdvancedCustomOptions/js/custom-options-type';
            $meta['custom_options']["children"]["options"]["children"]["record"]["children"]["container_option"]["children"]["values"]["children"]["record"]["children"][self::FIELD_OPTION_DEFAULT_NAME] = $this->getOptionDefaultConfig(56);
            $meta['custom_options']["children"]["options"]["children"]["record"]["children"]["container_option"]["children"]["values"]["children"]["record"]["children"][self::FIELD_OPTION_SALABLE_NAME] = $this->getOptionSalableConfig(57);
            $meta['custom_options']["children"]["options"]["children"]["record"]["children"]["container_option"]["children"][self::FB_CUSTOM_INDEX_IMAGE] = $meta['custom_options']["children"]["options"]["children"]["record"]["children"]["container_option"]["children"]["values"];
            $meta['custom_options']["children"]["options"]["children"]["record"]["children"]["container_option"]["children"][self::FB_CUSTOM_INDEX_IMAGE]["children"]["record"]["children"][self::FIELD_OPTION_IMAGE_NAME] = $this->getOptionImageFieldConfig(59);
        } catch (\Exception $e) {
            $e->getMessage();
        }

        return $meta;
    }

    /**
     * @param array $data
     * @return array
     */
    public function modifyData(array $data)
    {
        //  $data = parent::modifyData($data);

        $product = $this->locator->getProduct();
        if ($product->getId()) {
            if ($product->getOptions() !== null) {
                $i = 0;
                foreach ($product->getOptions() as $option) {
                    $j = 0;
                    if ($option->getValues() !== null) {
                        foreach ($option->getValues() as $value) {
                            if ($value->getOptionImage()) {
                                $img = $value->getOptionImage();
                                $fullImagePath = $this->getImagePath($img);
                                try {
                                    if (file_exists($fullImagePath)) {
                                        $imageObject = [(object)[
                                            'name' => $img,
                                            'file' => $this->mediaFolderRelativePath . $img,
                                            'url'  => $this->getImageUrl($img),
                                            'size' => $this->getStat($img),
                                            'type' => $this->getMimeType($img)
                                        ]];
                                    } else {
                                        $imageObject = [(object)[]];
                                    }
                                } catch (NoSuchEntityException $e) {
                                } catch (LocalizedException $e) {
                                }

                                $data[$product->getId()]['product']['options'][$i]['values'][$j]['option_image'] = $imageObject;
                                $data[$product->getId()]['product']['options'][$i]['values'][$j]['default_image'] = $imageObject;
                                $data[$product->getId()]['product']['options'][$i]['values'][$j]['store_image'] = $imageObject;
                            }
                            $j++;
                        }
                    }
                    $i++;
                }

                $k = 0;
                foreach ($product->getOptions() as $option) {
                    //If image_option we need to put the type in it's config
                    if (in_array($option->getType(), [self::FIELD_OPTION_IMAGE_NAME]) && isset($data[$product->getId()]['product']['options'][$k]['values'])) {
                        $data[$product->getId()]['product']['options'][$k][self::FB_CUSTOM_INDEX_IMAGE] = $data[$product->getId()]['product']['options'][$k]['values'];
                        unset($data[$product->getId()]['product']['options'][$k]['values']);
                    }
                    $k++;
                }
            }
        }

        return $data;
    }
}
